#include <iostream>
#include <stdio.h>
#include <Application.h> 
#include <Window.h> 
#include <Bitmap.h>
#include <View.h>
#include <Rect.h>
#include <Point.h>
#include "Picture.h"
#include <MenuItem.h>
#include <MenuField.h>
#include <PopUpMenu.h>
#include <String.h>
#include <unistd.h>
#include <Roster.h>
#include <Button.h>
#include <StringView.h>
#include <StorageKit.h>
#include <Path.h>
#include <FindDirectory.h>
#include <stdlib.h>

#include <Screen.h>

#define BUTTON_OK 'OK'
#define BUTTON_CANC 'CANC'

const uint32 MENU_ITEM_PICKED_SHUTDOWN = 'Stch';
const uint32 MENU_ITEM_PICKED_RESTART = 'Rtch';
const uint32 MENU_ITEM_PICKED_LOCK = 'Ltch';
const uint32 MENU_ITEM_PICKED_STANDBY = 'Dtch';
const uint32 MENU_ITEM_PICKED_SCREENSVR = 'Vtch';


const int SHUT_DOWN = 1;
const int RESTART = 2;
const int LOCK = 3;
const int STANDBY = 4;
const int SCREENSVR = 5;

char* instructs[5] = {"Shuts down BeOS so that you can safely turn off power.",
							"Shuts down BeOS and starts BeOS again.",
							"Locks this BeOS Workstation.",
							"Maintains your session on low power with data still in memory",
							"Starts your current screen saver."};

int iSelection = -1;

BPath mpath;
BBitmap* screenshot; 

class PictureView : public BView
{
	public:
		PictureView(BBitmap* bitmap, BRect r);
		virtual void Draw(BRect drawRect);
	private:
		BBitmap* drawme;
};

PictureView::PictureView(BBitmap* bitmap, BRect r) : BView(r,"pictureview",B_FOLLOW_NONE,B_WILL_DRAW)
{
	drawme = bitmap;
}

void PictureView::Draw(BRect drawRect)
{
	DrawBitmap(drawme,BPoint(0,0));
}

int GetLastSetting()
{
	/* taken from Marius Stene's LockWorkstation source */
	// look for settings file, if it doesn't exist then create it with the default SHUT_DOWN option
	find_directory(B_USER_SETTINGS_DIRECTORY, &mpath);
	mpath.Append("EndSession.BeOS_settings");
	FILE * loadfile = fopen(mpath.Path(), "r");
	if (!loadfile)
	{
			fclose(loadfile);
			FILE* savefile = fopen(mpath.Path(), "w");
			//if can't open the file set the default as SHUT_DOWN
			fprintf(savefile,"%d",SHUT_DOWN);
			fclose(savefile);
			loadfile = fopen(mpath.Path(), "r");
	}
	
	//get the last-selected choice from the settings file
	char previous_choice[1];
	fgets(previous_choice, 1, loadfile);
	fclose(loadfile);
	return atoi(previous_choice);
}

void SaveLastChoice(int iLastChoice)
{
	// taken from Marius Stene's LockWorkstation source 
	find_directory(B_USER_SETTINGS_DIRECTORY, &mpath);
	mpath.Append("EndSession.BeOS_settings");
	FILE * savefile = fopen(mpath.Path(), "w");
	//write the current setting..
	fprintf(savefile,"%d",iLastChoice);
	fclose(savefile);
}

class PictureWindow : public BWindow
{
	public:
		PictureWindow();
		virtual bool QuitRequested();
		virtual void MessageReceived(BMessage* msg);
	private:
		BStringView* lblInstruction;
		BStringView* lblOutput;
		BPopUpMenu* UserChoice;
		BButton*		btnOK;
		BButton*		btnCANCEL;
	
		BBitmap* source;
		PictureView* PVsource;
	
};

PictureWindow :: PictureWindow()
	: BWindow(BRect(350,200,759,440), "Shut Down BeOS", B_TITLED_WINDOW, B_NOT_ZOOMABLE|B_NOT_RESIZABLE|B_NOT_MINIMIZABLE|B_NOT_MOVABLE)
{
	BView* parent = new BView(Bounds(),"parent",B_FOLLOW_ALL,B_WILL_DRAW);
	parent->SetDrawingMode(B_OP_OVER);
	parent->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));

	AddChild(parent);

	BBitmap* image = new BBitmap(BRect(0,0,249,105),B_RGB32);
	image->SetBits(beos_logo,79500,0,B_RGB32);

	PictureView* BeOSView = new PictureView(image,BRect(0,0,415,104));
	// PictureView* BeOSView = new PictureView(image,BRect(83,0,332,105)); // centered


	// set the background to white only because the Picture.h image is white
	BeOSView->SetViewColor(255,255,255);
	
	BeOSView->SetDrawingMode(B_OP_OVER);
	
	parent->AddChild(BeOSView);
	
	//////////////////////////////////
	// add icon on the left...
	//////////////////////////////////
	
	BFile file("/boot/apps/EndSession.BeOS",B_READ_ONLY);
	BAppFileInfo app_info(&file);
	
	BBitmap* bitmap = new BBitmap(BRect(0,0,31,31),B_CMAP8);
	app_info.GetIcon(bitmap,B_LARGE_ICON);
	
	//PVsource = new PictureView(bitmap,BRect(10,125,41,156));
	PVsource = new PictureView(bitmap,BRect(20,125,61,156));
	PVsource->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	PVsource->SetDrawingMode(B_OP_ALPHA); 
	parent->AddChild(PVsource);
	
	
	//////////////////////////////////
	// end add icon on the left...
	//////////////////////////////////
	
    UserChoice = new BPopUpMenu("Pick: ");
	BString label;
	label = "Shut down";
	UserChoice->AddItem(new BMenuItem(label.String(),new BMessage(MENU_ITEM_PICKED_SHUTDOWN)));
	label = "Restart";
	UserChoice->AddItem(new BMenuItem(label.String(),new BMessage(MENU_ITEM_PICKED_RESTART)));
	label = "Lock Workstation";
	UserChoice->AddItem(new BMenuItem(label.String(),new BMessage(MENU_ITEM_PICKED_LOCK)));
	label = "Stand by";
	UserChoice->AddItem(new BMenuItem(label.String(),new BMessage(MENU_ITEM_PICKED_STANDBY)));
	label = "Screen Saver";
	UserChoice->AddItem(new BMenuItem(label.String(),new BMessage(MENU_ITEM_PICKED_SCREENSVR)));
	
	//where we want the first item to be
	BRect items(85,135,325,155);
	//Add the menufields to the view
	parent->AddChild(new BMenuField(items,"_UserChoice",NULL,UserChoice,true));  // true - doesn't adjust it's size to width of the items in the menu
	items.OffsetBy(0,130);
	
	lblInstruction = new BStringView(BRect(85,105,437,135),"_lblInstruction","What do you want the computer to do?",B_FOLLOW_LEFT | B_FOLLOW_TOP,B_WILL_DRAW);
	
	parent->AddChild(lblInstruction);

	btnOK = new BButton(BRect(240,207,320,232), "_btnOK", "OK",new BMessage(BUTTON_OK));					
	parent->AddChild(btnOK);
	
	btnOK->MakeDefault(true);  // automatically responds to user press of Enter key
	
	btnCANCEL = new BButton(BRect(324,207,407,232), NULL, "Cancel", new BMessage(BUTTON_CANC));
	parent->AddChild(btnCANCEL);
	
	lblOutput =new BStringView(BRect(85,160,437,180),"_lblOutput","",B_FOLLOW_LEFT | B_FOLLOW_TOP,B_WILL_DRAW);
	parent->AddChild(lblOutput);
	
	iSelection = GetLastSetting();
	UserChoice->ItemAt(iSelection-1)->SetMarked(true);
	lblOutput->SetText(instructs[iSelection-1]);

	
}

void PictureWindow::MessageReceived(BMessage* msg)
{

	switch(msg->what)
	{
		case BUTTON_OK:
			
			printf("iSelection is: %d\n", iSelection);
			switch(iSelection)
			{
				case SHUT_DOWN:
				printf("Shut down selected\n");
				system("/boot/beos/bin/shutdown");
				break;
				
				case RESTART:
				printf("Restart selected\n");
				system("/boot/beos/bin/shutdown -r");
				break;
				
				case LOCK:
				printf("Lock selected\n");
				system("LockWorkstation &");
				break;
				
				case STANDBY:
				printf("Standby selected\n");
				system("apmctl standby &");
				break;
				
				case SCREENSVR:
				printf("ScreenSaver selected\n");
				system("screen_blanker &");
				break;
				
				default:
				printf("nothing selected\n");
			}	
			
			if (iSelection != -1)
			{
				SaveLastChoice(iSelection);
				be_app->PostMessage(B_QUIT_REQUESTED);// tell the application to quit too
			}
		break;
	
		case BUTTON_CANC:
			be_app->PostMessage(B_QUIT_REQUESTED);// tell the application to quit too
			
		break;
		
		case MENU_ITEM_PICKED_SHUTDOWN:
		{
			lblOutput->SetText(instructs[SHUT_DOWN-1]);
			iSelection = SHUT_DOWN;
		}
		break; 
		
		case MENU_ITEM_PICKED_RESTART:
		{
			lblOutput->SetText(instructs[RESTART-1]);
			iSelection = RESTART;
		}
		break; 
		
		case MENU_ITEM_PICKED_LOCK:
		{
			lblOutput->SetText(instructs[LOCK-1]);
			iSelection = LOCK;
		}
		break; 
	
		case MENU_ITEM_PICKED_STANDBY:
		{
			lblOutput->SetText(instructs[STANDBY-1]);
			iSelection = STANDBY;
		}
		break; 
	
		case MENU_ITEM_PICKED_SCREENSVR:
		{
			lblOutput->SetText(instructs[SCREENSVR-1]);
			iSelection = SCREENSVR;
		}
		break; 
		
//		I need to implement the esc key press for quit 
//		
//		case B_KEY_DOWN:
//		{
//			printf("key press\n");
//		}
		
		
		default:
			BWindow::MessageReceived(msg);
	}

}

// Called when the user clicks on the window's close box
bool PictureWindow :: QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);// tell the application to quit too
	return true; // and indicate that it's okay for us to close
}

class PictureWindowBG : public BWindow
{
	public:
		PictureWindowBG();
		virtual bool QuitRequested();
};
PictureWindowBG :: PictureWindowBG()
	: BWindow(BRect(0,0,1600,1200), "Shut Down BeOS", B_BORDERED_WINDOW,B_NOT_ZOOMABLE|B_NOT_RESIZABLE)
{
	
// this is new from tb100 (aka Simon) to create semi-transparent background..
 //original working version...
	PictureView* BeOSViewBG = new PictureView(screenshot,BRect(0,0,1600,1200));
	BeOSViewBG->SetViewColor(0,0,60);
	BeOSViewBG->SetDrawingMode(B_OP_ALPHA);
	BeOSViewBG->SetBlendingMode(B_CONSTANT_ALPHA, B_ALPHA_OVERLAY); 
	BeOSViewBG->SetHighColor(0, 0, 0, 95); ; //Last number is opacity, 0-255 
	//View set up, now draw 
	BeOSViewBG->FillRect(BRect(0, 0, 1599, 1199));
	AddChild(BeOSViewBG);

}

// Called when the user clicks on the window's close box
bool PictureWindowBG :: QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);// tell the application to quit too
	return true; // and indicate that it's okay for us to close
}

int main(int argc, char ** argv)
{
	BApplication app("application/x-vnd.EndSession.BeOS");//gotta have this
	//Create the window..
	BWindow* window = new PictureWindow();
	
	//// get screen capture of whole screen ...
		BScreen* oScreenObj = new BScreen(window);
		screenshot = new BBitmap(BRect(0,0,1600,1200),B_RGB32);
		oScreenObj->ReadBitmap(screenshot,false); //,BRect(0,0,1600,1200));
	
	BWindow* windowBG = new PictureWindowBG();
	
	windowBG->SetFeel(B_MODAL_APP_WINDOW_FEEL);
	windowBG->SetFlags(B_AVOID_FOCUS);
	windowBG->SetFlags(B_AVOID_FRONT);
	windowBG->Show();
	window->SetFeel(B_MODAL_ALL_WINDOW_FEEL);
	window->Show();	//show the window
	app.Run();
	return 0;
}



